@@ -25,6 +25,9 @@ from twisted.conch import interfaces as iconch
 from epsilon import extime, process, cooperator, modal, juice
 
 from axiom import iaxiom, errors as eaxiom, item, attributes
+from axiom.scheduler import Scheduler, SubScheduler
+from axiom.upgrade import registerUpgrader
+from axiom.dependency import installOn
 
 VERBOSE = False
 
@@ -270,6 +273,15 @@ class _ReliableListener(item.Item):
 
 
 class _BatchProcessorMixin:
+
+    def installed(self):
+        # XXX This is kind of suboptimal because it circumvents the usual
+        # dependency mechanism. But see #1408.
+
+        sch = iaxiom.IScheduler(self.store, None)
+        if sch is None:
+            installOn(SubScheduler(store=self.store), self.store)
+
     def step(self, style=iaxiom.LOCAL, skip=()):
         now = extime.Time()
         first = True
@@ -300,10 +312,21 @@ class _BatchProcessorMixin:
 
 
     def run(self):
+        """
+        Try to run one unit of work through one listener.  If there are more
+        listeners or more work, reschedule this item to be run again in
+        C{self.busyInterval} milliseconds, otherwise unschedule it.
+
+        @rtype: L{extime.Time} or C{None}
+        @return: The next time at which to run this item, used by the scheduler
+        for automatically rescheduling, or None if there is no more work to do.
+        """
         now = extime.Time()
         if self.step():
-            return now + datetime.timedelta(milliseconds=self.busyInterval)
-        return now + datetime.timedelta(milliseconds=self.idleInterval)
+            self.scheduled = now + datetime.timedelta(milliseconds=self.busyInterval)
+        else:
+            self.scheduled = None
+        return self.scheduled
 
 
     def timedEventErrorHandler(self, timedEvent, failureObj):
@@ -345,6 +368,10 @@ class _BatchProcessorMixin:
             forwardMark = 0
             backwardMark = 0
 
+        if self.scheduled is None:
+            self.scheduled = extime.Time()
+            iaxiom.IScheduler(self.store).schedule(self, self.scheduled)
+
         return _ReliableListener(store=self.store,
                                  processor=self,
                                  listener=listener,
@@ -384,20 +411,77 @@ class _BatchProcessorMixin:
             yield (failed.listener, failed.item)
 
 
+    def itemAdded(self):
+        """
+        Called to indicate that a new item of the type monitored by this batch
+        processor is being added to the database.
+
+        If this processor is not already scheduled to run, this will schedule
+        it.
+        """
+        localCount = self.store.query(
+            _ReliableListener,
+            attributes.AND(_ReliableListener.processor == self,
+                           _ReliableListener.style == iaxiom.LOCAL),
+            limit=1).count()
+
+        if localCount and self.scheduled is None:
+            self.scheduled = extime.Time()
+            iaxiom.IScheduler(self.store).schedule(self, self.scheduled)
+
+
+
+def upgradeProcessor1to2(oldProcessor):
+    """
+    Batch processors stopped polling at version 2, so they no longer needed the
+    idleInterval attribute.  They also gained a scheduled attribute which
+    tracks their interaction with the scheduler.  Since they stopped polling,
+    we also set them up as a timed event here to make sure that they don't
+    silently disappear, never to be seen again: running them with the scheduler
+    gives them a chance to figure out what's up and set up whatever other state
+    they need to continue to run.
+
+    Since this introduces a new dependency of all batch processors on a powerup
+    for the IScheduler, install a Scheduler or a SubScheduler if one is not
+    already present.
+    """
+    newProcessor = oldProcessor.upgradeVersion(
+        oldProcessor.typeName, 1, 2,
+        busyInterval=oldProcessor.busyInterval)
+    newProcessor.scheduled = extime.Time()
+
+    s = newProcessor.store
+    sch = iaxiom.IScheduler(s, None)
+    if sch is None:
+        if s.parent is None:
+            # Only site stores have no parents.
+            sch = Scheduler(store=s)
+        else:
+            # Substores get subschedulers.
+            sch = SubScheduler(store=s)
+        installOn(sch, s)
+
+    # And set it up to run.
+    sch.schedule(newProcessor, newProcessor.scheduled)
+    return newProcessor
 
 def processor(forType):
     """
-    Create an Axiom Item type which is suitable to use as a batch
-    processor for the given Axiom Item type.
+    Create an Axiom Item type which is suitable to use as a batch processor for
+    the given Axiom Item type.
+
+    Processors created this way depend on a L{iaxiom.IScheduler} powerup on the
+    on which store they are installed.
 
     @type forType: L{item.MetaItem}
-    @param forType: The Axiom Item type for which to create a batch
-    processor type.
+    @param forType: The Axiom Item type for which to create a batch processor
+    type.
 
     @rtype: L{item.MetaItem}
-    @return: An Axiom Item type suitable for use as a batch processor.
-    If such a type previously existed, it will be returned.
-    Otherwise, a new type is created.
+
+    @return: An Axiom Item type suitable for use as a batch processor.  If such
+    a type previously existed, it will be returned.  Otherwise, a new type is
+    created.
     """
     MILLI = 1000
     if forType not in _processors:
@@ -414,17 +498,27 @@ def processor(forType):
 
             '__repr__': lambda self: '<Batch of %s #%d>' % (reflect.qual(self.workUnitType), self.storeID),
 
+            'schemaVersion': 2,
+
             'workUnitType': forType,
 
+            'scheduled': attributes.timestamp(doc="""
+            The next time at which this processor is scheduled to run.
+            """, default=None),
+
             # MAGIC NUMBERS AREN'T THEY WONDERFUL?
-            'rate': attributes.integer(doc="", default=10),
-            'idleInterval': attributes.integer(doc="", default=60 * MILLI),
             'busyInterval': attributes.integer(doc="", default=MILLI / 10),
             }
         _processors[forType] = item.MetaItem(
             attrs['__name__'],
             (item.Item, _BatchProcessorMixin),
             attrs)
+
+        registerUpgrader(
+            upgradeProcessor1to2,
+            _processors[forType].typeName,
+            1, 2)
+
     return _processors[forType]
 
 
@@ -1102,7 +1196,7 @@ class BatchProcessingService(service.Service):
                 else:
                     if itemHasMore:
                         more = True
-                yield None, more
+                yield None, bool(more or items)
             if not ran:
                 yield None, more
 
@@ -1123,10 +1217,9 @@ if manhole_ssh is not None:
 else:
     bases = ()
 
-class BatchManholePowerup(item.Item, item.InstallableMixin):
-    #inheritance is a design pattern!@#
-
+class BatchManholePowerup(item.Item):
     installedOn = attributes.reference()
+
     original = attributes.inmemory()
     transportFactory = attributes.inmemory()
     chainedProtocolFactory = attributes.inmemory()
@@ -1136,16 +1229,12 @@ class BatchManholePowerup(item.Item, item.InstallableMixin):
     height = attributes.inmemory()
     conn = attributes.inmemory()
 
-    def installOn(self, other):
-        item.InstallableMixin.installOn(self, other)
-        other.powerUp(self, iconch.IConchUser)
-        other.powerUp(self, iconch.ISession)
-
+    powerupInterfaces = (iconch.IConchUser, iconch.ISession)
 
     def activate(self):
         if manhole_ssh is not None:
-            manhole_ssh.TerminalSession.__init__(self, self.installedOn)
-            manhole_ssh.TerminalUser.__init__(self, self.installedOn, None)
+            manhole_ssh.TerminalSession.__init__(self, self.store)
+            manhole_ssh.TerminalUser.__init__(self, self.store, None)
 
 
     def chainedProtocolFactory(self):
